use clap::Parser;
use std::fs;
use std::path::{Path, PathBuf};
use std::process::Command;
use std::time::UNIX_EPOCH;
use std::env;
// use std::time::{SystemTime, UNIX_EPOCH};

#[derive(Parser, Debug)]
#[command(
    author,
    version,
    about = "Convert source directory with odt files to target path with pdf files with changes verification"
)]
struct Args {
    #[arg(help = "Source directory with .odt files")]
    source: PathBuf,

    #[arg(help = "Target directory for PDFs converted files")]
    dest: PathBuf,
}

fn needs_conversion(odt_path: &Path, pdf_path: &Path) -> bool {
    if !pdf_path.exists() {
        return true;
    }

    let odt_modified = fs::metadata(odt_path)
        .and_then(|m| m.modified())
        .unwrap_or(UNIX_EPOCH);

    let pdf_modified = fs::metadata(pdf_path)
        .and_then(|m| m.modified())
        .unwrap_or(UNIX_EPOCH);

    odt_modified > pdf_modified
}

fn get_soffice_path() -> Result<String, String> {
    // First try environment variable if set
    if let Ok(path) = env::var("LIBREOFFICE_PATH") {
        if Path::new(&path).exists() {
            return Ok(path);
        }
    }

    // Check common installation paths
    let paths = if cfg!(target_os = "macos") {
        vec![
            "/Applications/LibreOffice.app/Contents/MacOS/soffice",
            "/Applications/LibreOffice-still.app/Contents/MacOS/soffice",
            "/Applications/LibreOffice-fresh.app/Contents/MacOS/soffice",
        ]
    } else {
        vec![
            "/usr/bin/soffice",
            "/usr/local/bin/soffice",
            "/usr/lib/libreoffice/program/soffice",
            "soffice",  // Try PATH
        ]
    };

    // Try each path
    for path in paths {
        if Path::new(path).exists() || which::which(path).is_ok() {
            return Ok(path.to_string());
        }
    }

    Err("LibreOffice not found. Please install LibreOffice or set LIBREOFFICE_PATH environment variable.".to_string())
}

fn convert_file(odt_path: &Path, dest_dir: &Path) -> Result<(), String> {
    let soffice_path = get_soffice_path()?;
    
    // Verify input file exists and is readable
    if !odt_path.exists() {
        return Err(format!("Source file does not exist: {}", odt_path.display()));
    }
    
    // Verify destination directory exists and is writable
    if !dest_dir.exists() {
        return Err(format!("Destination directory does not exist: {}", dest_dir.display()));
    }
    
    // Try to get write permissions on destination directory
    let metadata = fs::metadata(dest_dir)
        .map_err(|e| format!("Failed to get destination directory metadata: {}", e))?; 
    
    #[cfg(unix)]
    use std::os::unix::fs::PermissionsExt;
    #[cfg(unix)]
    if metadata.permissions().mode() & 0o200 == 0 {
        return Err(format!("Destination directory is not writable: {}", dest_dir.display()));
    }

    let output = Command::new(&soffice_path)
        .args(&[
            "--headless",
            "--convert-to",
            "pdf",
            "--outdir",
            dest_dir.to_str().ok_or("Invalid destination path")?,
            odt_path.to_str().ok_or("Invalid source path")?,
        ])
        .output()
        .map_err(|e| format!("Failed to execute LibreOffice command: {}", e))?;

    if !output.status.success() {
        let error_msg = String::from_utf8_lossy(&output.stderr);
        let stdout_msg = String::from_utf8_lossy(&output.stdout);
        return Err(format!(
            "Conversion failed:\nError: {}\nOutput: {}",
            error_msg,
            stdout_msg
        ));
    }

    Ok(())
}

fn main() {
    let args = Args::parse();

    // Create target directory if not exists
    if let Err(e) = fs::create_dir_all(&args.dest) {
        eprintln!("Error create target directory: {}", e);
        return;
    }

    // Process .odt files
    if let Ok(entries) = fs::read_dir(&args.source) {
        for entry in entries.flatten() {
            let path = entry.path();
            if path.extension().map_or(false, |ext| ext == "odt") {
                let pdf_path = args.dest.join(
                    path.file_stem()
                        .map(|s| s.to_string_lossy() + ".pdf")
                        .unwrap_or_default()
                        .as_ref(),
                );
                if needs_conversion(&path, &pdf_path) {
                    match convert_file(&path, &args.dest) {
                        Ok(_) => println!("Converted: {:?}", path.file_name().unwrap_or_default()),
                        Err(e) => eprintln!("Error: {} - {}", path.display(), e),
                    }
                }
            }
        }
    }
}
